#Python version '%' ==> '#' and 'np.' before trig functions
# Also ^ is replaced by ** for exponentiation
#Also 'nan' ==> 'np.nan' but we're going to replace with
#numbers anyway  
import numpy as np

# Using Matlab for a two-stage compound gear train example
# Everything marked Change will need to be entered manually
# from catalog data, tables, etc. and possibly changed 
# for each trial.
# In this blank version, everything set to 'nan' will be 
# entered manually.

################################
#Handy constants, conversion factors
m_to_in = 39.37                     #Meters to inches
N_to_lbf = 0.2248                   #Newtons to pounds force
Nm_to_inlbf = N_to_lbf*m_to_in      #Nm to inchpounds torque
inprsec_ftprmin = 60.0/12           #inches/second to feet/minute
rpm_to_radps = 2*np.pi/60           #rotations/minute to radians/second
deg_to_rad = np.pi/180              #degrees to radians
watt_to_hp = 745.7                  #Watts to horsepower
###############################

###############################
#Input requirements for the problem
Tmotor = 120               #inch lbf
Rpm    = 1000              #rpm of input

#Input assumptions about application (mounting, shock, etc.)
#See gears notes for details
#Set to 1.0 initially - >> CHANGE these!
Km =  np.nan # Mounting factor (rigidity, accuracy) 1.3<Km<2.3
Ko =  np.nan # Overload, shock 1.0<Ko<2.3

###############################
## Try some gears:
## First Stage - enter values from catalog
## CHANGE these with each new trial!
Pressure_angle = 20.0 #degrees pressure angle of gears
phi = deg_to_rad*Pressure_angle      
pitch1 = np.nan
Qv1 = np.nan         
Stage1N1 = np.nan 
Stage1N2 = np.nan
FaceWidth1 = np.nan
Jlewis1_1 =  np.nan # Look up from chart, changes with N1, N2
Jlewis1_2 =  np.nan # Look up from chart, changes with N1, N2

## Second Stage - enter values from catalog
## CHANGE these with each new trial
## Assume pressure angle same as 1st stage
pitch2 = np.nan
Qv2 = np.nan
Stage2N1 = np.nan
Stage2N2 = np.nan
FaceWidth2 = np.nan
Jlewis2_1 =  np.nan # Look up from chart, changes with N1, N2
Jlewis2_2 =  np.nan # Look up from chart, changes with N1, N2

##Check total speed ratio and torque, power, etc. Looks OK?
Omega_m = Rpm*rpm_to_radps               #omega motor in radians/second
InWatts = (Tmotor/Nm_to_inlbf)*Omega_m   #motor power in watts
ratio = (Stage1N2/Stage1N1)*(Stage2N2/Stage2N1)
Torque_out = Tmotor*ratio                #Want well above 720 inch-lbf
Omega_out = Omega_m/ratio
cycles_per_second = (Rpm/ratio)/60       #Want 1 to 2 cycles per sec


#######Part I: Bending Stress & Contact Ratio ########
## Compute tangential velocities, Kv factors, etc.
# Stage1
rp1 = 0.5*Stage1N1/pitch1
rp2 = 0.5*Stage1N2/pitch1
Ftan1 = Tmotor/rp1  # Tangential tooth force in lbf 
Vtan1 = rp1*Omega_m # Tangential velocity in inches/second
B1 = 0.25*(12.0-Qv1)**(2.0/3.0)
if Qv1 > 12:
    B1 = 0

A1 = 50.0+56.0*(1.0-B1)
Kv1 = ((A1+np.sqrt(Vtan1*inprsec_ftprmin))/A1)**B1 #velocity factor
SigmaLewis1 = (Ftan1*pitch1*Ko*Km*Kv1)/(FaceWidth1*Jlewis1_1)
SigmaLewis2 = (Ftan1*pitch1*Ko*Km*Kv1)/(FaceWidth1*Jlewis1_2)

# Stage2 (accounting for gear ratios)
rp3 = 0.5*Stage2N1/pitch2
rp4 = 0.5*Stage2N2/pitch2
Ftan2 = Ftan1*(rp2/rp3)      # Tangential tooth force, lbf 
Vtan2 = Vtan1*(rp3/rp2) # Tangential velocity, inches/second
B2 = 0.25*(12.0-Qv2)**(2.0/3.0)
if Qv2 > 12:
    B2 = 0

A2 = 50.0+56.0*(1.0-B2)
Kv2 = ((A2+np.sqrt(Vtan2*inprsec_ftprmin))/A2)**B2 #velocity factor
SigmaLewis3 = (Ftan2*pitch2*Ko*Km*Kv2)/(FaceWidth2*Jlewis2_1)
SigmaLewis4 = (Ftan2*pitch2*Ko*Km*Kv2)/(FaceWidth2*Jlewis2_2)

## Contact ratios (want > 1.5, if possible)
# Stage 1 contact ratio
C = 0.5*(Stage1N1/pitch1 + Stage1N2/pitch1)
Pbase = np.pi * np.cos(phi)/pitch1
ra1 = 0.5*(Stage1N1+2)/pitch1
ra2 = 0.5*(Stage1N2+2)/pitch1
rb1 = rp1*np.cos(phi)
rb2 = rp2*np.cos(phi)

numerator = np.sqrt(ra1**2-rb1**2) + np.sqrt(ra2**2-rb2**2) - C*np.sin(phi)
ContactRatio1 = numerator/Pbase

# Stage 2 contact ratio
C = 0.5*(Stage2N1/pitch2 + Stage2N2/pitch2)
Pbase = np.pi * np.cos(phi)/pitch2
ra3 = 0.5*(Stage2N1+2)/pitch2
ra4 = 0.5*(Stage2N2+2)/pitch2
rb3 = rp3*np.cos(phi)
rb4 = rp4*np.cos(phi)

numerator = np.sqrt(ra3**2-rb3**2) + np.sqrt(ra4**2-rb4**2) - C*np.sin(phi)
ContactRatio2 = numerator/Pbase


## Part II: Compute allowable lifetime stresses

Sigma_ult = np.nan  # Psi CHANGE to suit your material
Output_cycles = np.nan  # CHANGE this to suit your application

## ##
Gear4_cycles = Output_cycles
Gear3_cycles = (Stage2N2/Stage2N1)*Output_cycles
Gear2_cycles = Gear3_cycles
Gear1_cycles = (Stage1N2/Stage1N1)*Gear2_cycles

## For each gear, depending on number of cycles, get Sigma_nprime
#Need to CHANGE factor & constants below based on material
factor = np.nan  #True for infinite life, steel. Else use Fig. 8.5
              #factor might be different for each gear!
Sigma_nprime = factor*Sigma_ult

Cg = 1.0   # For bending loads
Cs = np.nan  # Look up in table Cs (CHANGE to suit)
Kr = np.nan # Table 15.3, depends on reliability (CHANGE to suit)
Kt = 1.0   # 1 unless high temp
Kms = 1.4  # 1.0 if is idler gear
Sigma_allow = Cg*Cs*Kr*Kt*Kms*Sigma_nprime

#For each gear compare with corresponding SigmaLewis stresses above


## Part III: Compute Contact Stresses
# Script assumes speed reducers (N2>N1 each stage)

Cp1 = np.nan             #(CHANGE) from table, based on material
Cp2 = np.nan             #(CHANGE) from table, based on material

R = Stage1N2/Stage1N1
I1 = (np.sin(phi)*np.cos(phi)/2)*R/(R+1.0)
Sigma_c1 = Cp1*np.sqrt((Ftan1*Ko*Kv1*Km)/(rp1*2.0*FaceWidth1*I1))

R = Stage2N2/Stage2N1
I2 = (np.sin(phi)*np.cos(phi)/2)*R/(R+1.0)
Sigma_c2 = Cp2*np.sqrt((Ftan2*Ko*Kv2*Km)/(rp3*2.0*FaceWidth2*I2))

#Compare Sigma_c1 and Sigma_c2 to allowable lifetime contact stress
#This will be higher than allowable lifetime bending stress Sigma_Z